@@ -37,6 +37,25 @@
 \\[add-change-log-entry] calls this function (if nil, `add-log-current-defun'
 instead) with no arguments.  It returns a string or nil if it cannot guess.")
 
+;;;###autoload
+(defvar add-log-full-name nil	; lemacs change
+  "*Full name of user, for inclusion in ChangeLog daily headers.
+This defaults to the value returned by the `user-full-name' function.")
+
+;; So that the dump-time value doesn't go into loaddefs.el with the autoload.
+(or add-log-full-name (setq add-log-full-name (user-full-name)))
+
+;;;###autoload
+(defvar add-log-mailing-address nil	; lemacs change
+  "*Electronic mail address of user, for inclusion in ChangeLog daily headers.
+This defaults to the value returned by `user-login-name', followed by
+an `@' character, followed by the value returned by `system-name'.")
+
+;; So that the dump-time value doesn't go into loaddefs.el with the autoload.
+(or add-log-mailing-address
+    (setq add-log-mailing-address
+	  (concat (user-login-name) "@" (system-name))))
+
 (defun change-log-name ()
   (or change-log-default-name
       (if (eq system-type 'vax-vms) "$CHANGE_LOG$.TXT" "ChangeLog")))
@@ -110,22 +129,18 @@ Fourth arg NEW-ENTRY non-nil means always create a new entry at the front;
 never append to an existing entry."
   (interactive (list current-prefix-arg
 		     (prompt-for-change-log-name)))
-  (let* ((full-name (if whoami
-			(read-string "Full name: " (user-full-name))
-                        (user-full-name)))
-	 ;; Note that some sites have room and phone number fields in
-	 ;; full name which look silly when inserted.  Rather than do
-	 ;; anything about that here, let user give prefix argument so that
-	 ;; s/he can edit the full name field in prompter if s/he wants.
-	 (login-name (if whoami
-			 (read-string "Login name: " (user-login-name))
-                         (user-login-name)))
-	 (site-name (if whoami
-			(read-string "Site name: " (system-name))
-                        (system-name)))
-	 (defun (funcall (or add-log-current-defun-function
-			     'add-log-current-defun)))
-	 paragraph-end entry)
+  (if whoami
+      (progn
+        (setq add-log-full-name (read-string "Full name: " add-log-full-name))
+	;; Note that some sites have room and phone number fields in
+	;; full name which look silly when inserted.  Rather than do
+	;; anything about that here, let user give prefix argument so that
+	;; s/he can edit the full name field in prompter if s/he wants.
+	(setq add-log-mailing-address
+	      (read-string "Mailing address: " add-log-mailing-address))))
+  (let ((defun (funcall (or add-log-current-defun-function
+			    'add-log-current-defun)))
+	paragraph-end entry)
 
     (setq file-name (find-change-log file-name))
 
@@ -147,12 +162,12 @@ never append to an existing entry."
     (goto-char (point-min))
     (if (looking-at (concat (regexp-quote (substring (current-time-string)
 						     0 10))
-			    ".* " (regexp-quote full-name)
-			    "  (" (regexp-quote login-name) "@"))
+			    ".* " (regexp-quote add-log-full-name)
+			    "  (" (regexp-quote add-log-mailing-address)))
 	(forward-line 1)
       (insert (current-time-string)
-	      "  " full-name
-	      "  (" login-name "@" site-name ")\n\n"))
+	      "  " add-log-full-name
+	      "  (" add-log-mailing-address ")\n\n"))
 
     ;; Search only within the first paragraph.
     (if (looking-at "\n*[^\n* \t]")
@@ -214,9 +229,9 @@ never append to an existing entry."
 ;;;###autoload
 (defun add-change-log-entry-other-window (&optional whoami file-name)
   "Find change log file in other window and add an entry for today.
-First arg (interactive prefix) non-nil means prompt for user name and site.
-Second arg is file name of change log.
-Interactively, with a prefix argument, the file name is prompted for."
+Optional arg (interactive prefix) non-nil means prompt for user name and site.
+Second arg is file name of change log.  \
+If nil, uses `change-log-default-name'."
   (interactive (if current-prefix-arg
 		   (list current-prefix-arg
 			 (prompt-for-change-log-name))))
