@@ -1,5 +1,5 @@
 /* Synchronous subprocess invocation for GNU Emacs.
-   Copyright (C) 1985, 1986, 1987, 1988, 1992, 1993
+   Copyright (C) 1985, 1986, 1987, 1988, 1992, 1993, 1994
    Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
@@ -21,7 +21,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "config.h"
 
 #include <stdio.h>		/* For sprintf */
-#include <signal.h>
+#if defined(IRIX)
+#include <errno.h>	/* From gparker@sni-usa.com for IRIX 4.0.5E */
+#endif
 #include <sys/errno.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -51,6 +53,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #endif
 
 #include "lisp.h"
+#include "intl.h"
 #include "commands.h"
 #include "buffer.h"
 #include "paths.h"
@@ -67,19 +70,17 @@ Lisp_Object Vexec_path, Vexec_directory, Vdata_directory;
 
 Lisp_Object Vshell_file_name;
 
-#ifndef MAINTAIN_ENVIRONMENT
-/* List of strings to append to front of environment of
-   all subprocesses when they are started.  */
-
+/* The environment to pass to all subprocesses when they are started.
+   This is in the semi-bogus format of ("VAR=VAL" "VAR2=VAL2" ... )
+ */
 Lisp_Object Vprocess_environment;
-#endif
 
 /* True iff we are about to fork off a synchronous process or if we
    are waiting for it.  */
 int synch_process_alive;
 
 /* Nonzero => this is a string explaining death of synchronous subprocess.  */
-const char *synch_process_death;
+CONST char *synch_process_death;
 
 /* If synch_process_death is zero,
    this is exit code of synchronous subprocess.  */
@@ -117,7 +118,7 @@ call_process_cleanup (fdpid)
 
     record_unwind_protect (call_process_kill, fdpid);
     /*>>>>>>> "c-G" -- need non-consing Single-key-description */
-    message ("Waiting for process to die...(type C-g again to kill it instantly)");
+    message (GETTEXT ("Waiting for process to die...(type C-g again to kill it instantly)"));
 
     immediate_quit = 1;
     QUIT;
@@ -128,7 +129,7 @@ call_process_cleanup (fdpid)
     XCONS (fdpid)->cdr = Qnil;
     unbind_to (speccount, Qnil);
 
-    message ("Waiting for process to die...done");
+    message (GETTEXT ("Waiting for process to die... done"));
   }
   synch_process_alive = 0;
   emacs_close (fd);
@@ -147,7 +148,7 @@ report_fork_error (string, data)
   if (errno >= 0 && errno < sys_nerr)
     errstring = build_string (sys_errlist[errno]);
   else
-    errstring = build_string ("undocumented error code");
+    errstring = build_string (GETTEXT ("undocumented error code"));
 
   /* System error messages are capitalized.  Downcase the initial. */
   XSTRING (errstring)->data[0] = DOWNCASE (XSTRING (errstring)->data[0]);
@@ -214,7 +215,7 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
      breaks everything in the presence of ange-ftp-visited files, so
      fuck it. */
     if (NILP (Ffile_accessible_directory_p (current_dir)))
-      report_file_error ("Setting current directory",
+      report_file_error (GETTEXT ("Setting current directory"),
                          Fcons (current_buffer->directory, Qnil));
 #endif /* 0 */
     UNGCPRO;
@@ -262,13 +263,15 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
   filefd = emacs_open ((char *)XSTRING (infile)->data, O_RDONLY, 0);
   if (filefd < 0)
     {
-      report_file_error ("Opening process input file", Fcons (infile, Qnil));
+      report_file_error (GETTEXT ("Opening process input file"),
+			 Fcons (infile, Qnil));
     }
 
   if (NILP (path))
     {
       emacs_close (filefd);
-      report_file_error ("Searching for program", Fcons (args[0], Qnil));
+      report_file_error (GETTEXT ("Searching for program"),
+			 Fcons (args[0], Qnil));
     }
   new_argv[0] = (char *) XSTRING (path)->data;
 
@@ -303,12 +306,7 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
     int logging_on = 0;
 #endif
 
-#ifdef MAINTAIN_ENVIRONMENT
-    env = (char **) alloca (size_of_current_environ ());
-    get_current_environ (env);
-#else
     env = environ;
-#endif /* MAINTAIN_ENVIRONMENT */
 
 #if 0  /* Some systems don't have sigblock.  */
     mask = sigblock (sigmask (SIGCHLD));
@@ -321,6 +319,12 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
     logging_on = cadillac_stop_logging ();
 #endif
 
+    /* These vars record information from process termination.
+       Clear them now before process can possibly terminate,
+       to avoid timing error if process terminates soon.  */
+    synch_process_death = 0;
+    synch_process_retcode = 0;
+
     fork_error = Qnil;
     pid = vfork ();
 
@@ -360,7 +364,7 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
   if (pid < 0)
     {
       emacs_close (fd[0]);
-      report_file_error ("Doing vfork", Qnil);
+      report_file_error (GETTEXT ("Doing vfork"), Qnil);
     }
 
   if (FIXNUMP (buffer))
@@ -374,9 +378,6 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
       return Qnil;
     }
 
-  synch_process_death = 0;
-  synch_process_retcode = 0;
-
   {
     Lisp_Object old = Fcurrent_buffer ();
     struct gcpro gcpro1;
@@ -452,7 +453,7 @@ child_setup (in, out, err, new_argv, env, set_pgrp, current_dir)
      char **new_argv;
      char **env;
      int set_pgrp;
-     const char *current_dir;
+     CONST char *current_dir;
 {
   register int pid = getpid ();
   register char *pwd;
@@ -501,7 +502,6 @@ child_setup (in, out, err, new_argv, env, set_pgrp, current_dir)
     }
   }
 
-#ifndef MAINTAIN_ENVIRONMENT
   /* Set `env' to a vector of the strings in Vprocess_environment.  */
   {
     register Lisp_Object tem;
@@ -515,27 +515,47 @@ child_setup (in, out, err, new_argv, env, set_pgrp, current_dir)
 	 tem = XCONS (tem)->cdr)
       new_length++;
 
-    /* new_length + 1 to include terminating 0 */
+    /* new_length + 1 to include terminating 0.  */
     env = new_env = (char **) alloca ((new_length + 1) * sizeof (char *));
 
-    /* Copy the env strings into new_env.  */
+    /* Copy the Vprocess_environment strings into new_env.  */
     for (tem = Vprocess_environment;
 	 (CONSP (tem)
 	  && STRINGP (XCONS (tem)->car));
 	 tem = XCONS (tem)->cdr)
     {
-      register char *e = (char *) XSTRING (XCONS (tem)->car)->data;
-      if (pwd && !strncmp ("PWD=", e, 4))
-      {
-        *new_env++ = pwd;
-        pwd = 0;
-      }
+      char **ep = env;
+      char *string = (char *) XSTRING (XCONS (tem)->car)->data;
+      /* See if this string duplicates any string already in the env.
+	 If so, don't put it in.
+	 When an env var has multiple definitions,
+	 we keep the definition that comes first in process-environment.  */
+      for (; ep != new_env; ep++)
+	{
+	  char *p = *ep, *q = string;
+	  while (1)
+	    {
+	      if (*q == 0)
+		/* The string is malformed; might as well drop it.  */
+		goto duplicate;
+	      if (*q != *p)
+		break;
+	      if (*q == '=')
+		goto duplicate;
+	      p++, q++;
+	    }
+	}
+      if (pwd && !strncmp ("PWD=", string, 4))
+	{
+	  *new_env++ = pwd;
+	  pwd = 0;
+	}
       else
-        *new_env++ = e;
+        *new_env++ = string;
+    duplicate: ;
     }
     *new_env = 0;
   }
-#endif /* Not MAINTAIN_ENVIRONMENT */
 
   /* Make sure that in, out, and err are not actually already in
      descriptors zero, one, or two; this could happen if Emacs is
@@ -556,13 +576,15 @@ child_setup (in, out, err, new_argv, env, set_pgrp, current_dir)
   emacs_close (out);
   emacs_close (err);
 
-#if !defined (IRIX)
-#if defined (USG)
+#ifndef IRIX
+# ifdef USG
+#  ifndef SETPGRP_RELEASES_CTTY
   setpgrp ();			/* No arguments but equivalent in this case */
-#else
+#  endif
+#  else
   setpgrp (pid, pid);
-#endif /* USG */
-#endif /* IRIX */
+#  endif /* USG */
+# endif /* IRIX */
   setpgrp_of_tty (pid);
 
 #ifdef vipc
@@ -575,8 +597,11 @@ child_setup (in, out, err, new_argv, env, set_pgrp, current_dir)
   environ = env;
   execvp (new_argv[0], new_argv);
 
-  emacs_write (1, "Couldn't exec the program ", 26);
-  emacs_write (1, new_argv[0], strlen (new_argv[0]));
+  {
+    char buf[255];
+    sprintf (buf, GETTEXT ("Couldn't exec the program %s"), new_argv[0]);
+    emacs_write (1, buf, strlen (buf));
+  }
   _exit (1);
 }
 
@@ -592,8 +617,8 @@ relocate_fd (int fd, int min)
       int new = dup (fd);
       if (new == -1)
 	{
-	  const char *message1 = "Error while setting up child: ";
-	  const char *message2 = "\n";
+	  CONST char *message1 = GETTEXT ("Error while setting up child: ");
+	  CONST char *message2 = "\n";
 	  emacs_write (2, message1, strlen (message1));
 	  emacs_write (2, sys_errlist[errno], strlen (sys_errlist[errno]));
 	  emacs_write (2, message2, strlen (message2));
@@ -606,6 +631,74 @@ relocate_fd (int fd, int min)
       return new;
     }
 }
+
+static int
+getenv_internal (CONST char *var,
+		 int varlen,
+		 char **value,
+		 int *valuelen)
+{
+  Lisp_Object scan;
+
+  for (scan = Vprocess_environment; CONSP (scan); scan = XCONS (scan)->cdr)
+    {
+      Lisp_Object entry = XCONS (scan)->car;
+      
+      if (XTYPE (entry) == Lisp_String
+	  && XSTRING (entry)->size > varlen
+	  && XSTRING (entry)->data[varlen] == '='
+	  && ! memcmp (XSTRING (entry)->data, var, varlen))
+	{
+	  *value    = (char *) XSTRING (entry)->data + (varlen + 1);
+	  *valuelen = XSTRING (entry)->size - (varlen + 1);
+	  return 1;
+	}
+    }
+
+  return 0;
+}
+
+DEFUN ("getenv", Fgetenv, Sgetenv, 1, 2, "sEnvironment variable: \np",
+  "Return the value of environment variable VAR, as a string.\n\
+VAR is a string, the name of the variable.\n\
+When invoked interactively, prints the value in the echo area.")
+     (var, interactivep)
+     Lisp_Object var, interactivep;
+{
+  char *value;
+  int valuelen;
+  Lisp_Object v = Qnil;
+  CHECK_STRING (var, 0);
+  if (getenv_internal ((char *) XSTRING (var)->data,
+		       XSTRING (var)->size,
+		       &value, &valuelen))
+    v = make_string (value, valuelen);
+  if (!NILP (interactivep))
+    {
+      if (NILP (v))
+	message (GETTEXT ("%s not defined in environment"),
+		 XSTRING (var)->data);
+      else
+	message ("\"%s\"", value);
+    }
+  return v;
+}
+
+/* A version of getenv that consults process_environment, easily
+   callable from C.  */
+char *
+egetenv (var)
+     CONST char *var;
+{
+  char *value;
+  int valuelen;
+
+  if (getenv_internal (var, strlen (var), &value, &valuelen))
+    return value;
+  else
+    return 0;
+}
+
 #endif /* not VMS */
 
 void
@@ -614,6 +707,17 @@ init_callproc ()
   register char * sh;
   Lisp_Object tempdir;
 
+  Vprocess_environment = Qnil;
+#ifndef CANNOT_DUMP
+  if (initialized)
+#endif
+    {
+      char **envp;
+      for (envp = environ; *envp; envp++)
+	Vprocess_environment = Fcons (build_string (*envp),
+				      Vprocess_environment);
+    }
+
   {
     char *data_dir = egetenv ("EMACSDATA");
     
@@ -652,7 +756,7 @@ init_callproc ()
     if (access ((char *) XSTRING (tempdir)->data, 0) < 0)
     {
       fprintf (stderr,
-               "Warning: machine-dependent data dir (%s) does not exist.\n",
+               GETTEXT ("Warning: machine-dependent data dir (%s) does not exist.\n"),
                XSTRING (Vexec_directory)->data);
       sleep (2);
     }
@@ -664,7 +768,7 @@ init_callproc ()
     if (access ((char *) XSTRING (tempdir)->data, 0) < 0)
     {
       fprintf (stderr,
-               "Warning: machine-independent data dir (%s) does not exist.\n",
+               GETTEXT ("Warning: machine-independent data dir (%s) does not exist.\n"),
                XSTRING (Vdata_directory)->data);
       sleep (2);
     }
@@ -676,21 +780,20 @@ init_callproc ()
   sh = (char *) egetenv ("SHELL");
   Vshell_file_name = build_string (sh ? sh : "/bin/sh");
 #endif
+}
+
+void
+set_process_environment ()
+{
+  register char **envp;
 
-#ifndef MAINTAIN_ENVIRONMENT
-  /* The equivalent of this operation was done
-     in init_environ in environ.c if MAINTAIN_ENVIRONMENT */
   Vprocess_environment = Qnil;
 #ifndef CANNOT_DUMP
   if (initialized)
 #endif
-    {
-      char **envp;
-      for (envp = environ; *envp; envp++)
-	Vprocess_environment = Fcons (build_string (*envp),
-				      Vprocess_environment);
-    }
-#endif /* MAINTAIN_ENVIRONMENT */
+    for (envp = environ; *envp; envp++)
+      Vprocess_environment = Fcons (build_string (*envp),
+				    Vprocess_environment);
 }
 
 void
@@ -712,15 +815,14 @@ especially executable programs intended for Emacs to invoke.");
     "Directory of architecture-independent files that come with GNU Emacs,\n\
 intended for Emacs to use.");
 
-#ifndef MAINTAIN_ENVIRONMENT
   DEFVAR_LISP ("process-environment", &Vprocess_environment,
     "List of environment variables for subprocesses to inherit.\n\
 Each element should be a string of the form ENVVARNAME=VALUE.\n\
 The environment which Emacs inherits is placed in this variable\n\
-when Emacs starts.")
-#endif
+when Emacs starts.");
 
 #ifndef VMS
   defsubr (&Scall_process);
+  defsubr (&Sgetenv);
 #endif
 }
