@@ -25,15 +25,16 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "buffer.h"
 #include "syntax.h"
 #include "insdel.h"
+#include "intl.h"
 
 Lisp_Object Qsyntax_table_p;
 
 int words_include_escapes;
 
+int parse_sexp_ignore_comments;
+  
 Lisp_Object Vstandard_syntax_table;
 
-/* There is an alist of syntax tables: names (strings) vs obarrays. */
-
 /* This is the internal form of the parse state used in parse-partial-sexp.  */
 
 struct lisp_parse_state
@@ -87,15 +88,15 @@ find_defun_start (pos)
     return find_start_value;
 
   /* Back up to start of line.  */
-  tem = scan_buffer ('\n', pos, -1, &shortage);
+  tem = scan_buffer (current_buffer, '\n', pos, -1, &shortage);
 
   while (tem > BEGV)
     {
       /* Open-paren at start of line means we found our defun-start.  */
-      if (SYNTAX (table, CHAR_AT (tem)) == Sopen)
+      if (SYNTAX (table, FETCH_CHAR (tem)) == Sopen)
 	break;
       /* Move to beg of previous line.  */
-      tem = scan_buffer ('\n', tem, -2, &shortage);
+      tem = scan_buffer (current_buffer, '\n', tem, -2, &shortage);
     }
 
   /* Record what we found, for the next try.  */
@@ -114,22 +115,20 @@ Any vector of 256 elements will do.")
   (obj)
      Lisp_Object obj;
 {
-  if (VECTORP (obj) && XVECTOR (obj)->size == 0400)
+  if (VECTORP (obj) && vector_length (XVECTOR (obj)) == 0400)
     return Qt;
   return Qnil;
 }
 
 static Lisp_Object
-check_syntax_table (obj)
-     Lisp_Object obj;
+check_syntax_table (Lisp_Object obj, Lisp_Object def)
 {
-  register Lisp_Object tem;
-  while (tem = Fsyntax_table_p (obj),
-	 NILP (tem))
+  if (NILP (obj))
+    obj = def;
+  while (NILP (Fsyntax_table_p (obj)))
     obj = wrong_type_argument (Qsyntax_table_p, obj);
-  return obj;
-}   
-
+  return (obj);
+}
 
 DEFUN ("syntax-table", Fsyntax_table, Ssyntax_table, 0, 0, 0,
   "Return the current syntax table.\n\
@@ -155,13 +154,12 @@ It is a copy of the TABLE, which defaults to the standard syntax table.")
      Lisp_Object table;
 {
   Lisp_Object val = make_vector (0400, Qzero);
-  if (!NILP (table))
-    table = check_syntax_table (table);
-  else if (NILP (Vstandard_syntax_table))
+
+  if (NILP (Vstandard_syntax_table))
     /* Can only be null during initialization */
     return val;
-  else
-    table = Vstandard_syntax_table;
+
+  table = check_syntax_table (table, Vstandard_syntax_table);
 
   memcpy (XVECTOR (val)->contents, XVECTOR (table)->contents,
 	  0400 * sizeof (Lisp_Object));
@@ -174,7 +172,7 @@ One argument, a syntax table.")
   (table)
      Lisp_Object table;
 {
-  table = check_syntax_table (table);
+  table = check_syntax_table (table, Qnil);
   current_buffer->syntax_table = table;
   /* Indicate that this buffer now has a specified syntax table.  */
   current_buffer->local_var_flags |=
@@ -186,7 +184,7 @@ One argument, a syntax table.")
  into the code it signifies.
  This is used by modify-syntax-entry, and other things. */
 
-const unsigned char syntax_spec_code[0400] =
+CONST unsigned char syntax_spec_code[0400] =
   { 0377, 0377, 0377, 0377, 0377, 0377, 0377, 0377,
     0377, 0377, 0377, 0377, 0377, 0377, 0377, 0377,
     0377, 0377, 0377, 0377, 0377, 0377, 0377, 0377,
@@ -211,7 +209,7 @@ const unsigned char syntax_spec_code[0400] =
 /* Indexed by syntax code, give the letter that describes it. */
 
 
-const unsigned char syntax_code_spec[13] =
+CONST unsigned char syntax_code_spec[13] =
   {
     ' ', '.', 'w', '_', '(', ')', '\'', '\"', '$', '\\', '/', '<', '>'
   };
@@ -227,11 +225,7 @@ syntax table.")
      Lisp_Object ch, table;
 {
   CHECK_FIXNUM (ch, 0);
-
-  if (NILP (table))
-    table = current_buffer->syntax_table;
-  else
-    table = check_syntax_table (table);
+  table = check_syntax_table (table, current_buffer->syntax_table);
 
   return make_number (syntax_code_spec[(int) SYNTAX (table,
                                                      0xFF & XINT (ch))]);
@@ -263,7 +257,7 @@ scan_words (from, count)
 	      immediate_quit = 0;
 	      return 0;
 	    }
-	  code = SYNTAX (table, CHAR_AT (from));
+	  code = SYNTAX (table, FETCH_CHAR (from));
 	  if (words_include_escapes
 	      && (code == Sescape || code == Scharquote))
 	    break;
@@ -274,7 +268,7 @@ scan_words (from, count)
       while (1)
 	{
 	  if (from == end) break;
-	  code = SYNTAX (table, CHAR_AT (from));
+	  code = SYNTAX (table, FETCH_CHAR (from));
 	  if (!(words_include_escapes
 		&& (code == Sescape || code == Scharquote)))
 	    if (code != Sword)
@@ -292,7 +286,7 @@ scan_words (from, count)
 	      immediate_quit = 0;
 	      return 0;
 	    }
-	  code = SYNTAX (table, CHAR_AT (from - 1));
+	  code = SYNTAX (table, FETCH_CHAR (from - 1));
 	  if (words_include_escapes
 	      && (code == Sescape || code == Scharquote))
 	    break;
@@ -303,7 +297,7 @@ scan_words (from, count)
       while (1)
 	{
 	  if (from == beg) break;
-	  code = SYNTAX (table, CHAR_AT (from - 1));
+	  code = SYNTAX (table, FETCH_CHAR (from - 1));
 	  if (!(words_include_escapes
 		&& (code == Sescape || code == Scharquote)))
 	    if (code != Sword)
@@ -318,7 +312,7 @@ scan_words (from, count)
   return from;
 }
 
-DEFUN ("forward-word", Fforward_word, Sforward_word, 1, 1, "p",
+DEFUN ("forward-word", Fforward_word, Sforward_word, 1, 1, "_p",
   "Move point forward ARG words (backward if ARG is negative).\n\
 Normally returns t.\n\
 If an edge of the buffer is reached, point is left there\n\
@@ -329,139 +323,141 @@ and nil is returned.")
   int val;
   CHECK_FIXNUM (count, 0);
 
-  if (!(val = scan_words (point, XINT (count))))
+  if (!(val = scan_words (PT, XINT (count))))
     {
       SET_PT (XINT (count) > 0 ? ZV : BEGV);
       return Qnil;
     }
   SET_PT (val);
-  zmacs_region_stays = 1;
   return Qt;
 }
 
 static int char_quoted (int);
-static void
-scan_sexps_forward (struct lisp_parse_state*, int, int, int, int, Lisp_Object);
+static void scan_sexps_forward (struct lisp_parse_state *,
+				int from, int end, 
+				int targetdepth, int stopbefore,
+				Lisp_Object oldstate,
+				int commentstop);
 
 static int
 find_start_of_comment (from, stop, mask)
     int from, stop, mask;
 {
-    int c;
-    enum syntaxcode code;
-    Lisp_Object table = current_buffer->syntax_table;
+  int c;
+  enum syntaxcode code;
+  Lisp_Object table = current_buffer->syntax_table;
 
-    /* Look back, counting the parity of string-quotes,
-       and recording the comment-starters seen.
-       When we reach a safe place, assume that's not in a string;
-       then step the main scan to the earliest comment-starter seen
-       an even number of string quotes away from the safe place.
-       
-       OFROM[I] is position of the earliest comment-starter seen
-       which is I+2X quotes from the comment-end.
-       PARITY is current parity of quotes from the comment end.  */
-    int parity = 0;
-    char my_stringend = 0;
-    int string_lossage = 0;
-    int comment_end = from;
-    int comstart_pos = 0;
-    int comstart_parity = 0;
-    int styles_match_p = 0;
+  /* Look back, counting the parity of string-quotes,
+     and recording the comment-starters seen.
+     When we reach a safe place, assume that's not in a string;
+     then step the main scan to the earliest comment-starter seen
+     an even number of string quotes away from the safe place.
+     
+     OFROM[I] is position of the earliest comment-starter seen
+     which is I+2X quotes from the comment-end.
+     PARITY is current parity of quotes from the comment end.  */
+  int parity = 0;
+  char my_stringend = 0;
+  int string_lossage = 0;
+  int comment_end = from;
+  int comstart_pos = 0;
+  int comstart_parity = 0;
+  int styles_match_p = 0;
 
-    /* At beginning of range to scan, we're outside of strings;
-       that determines quote parity to the comment-end.  */
-    while (from != stop)
+  /* At beginning of range to scan, we're outside of strings;
+     that determines quote parity to the comment-end.  */
+  while (from != stop)
     {
-	/* Move back and examine a character.  */
-	from--;
+      /* Move back and examine a character.  */
+      from--;
 
-	c = CHAR_AT (from);
-	code = SYNTAX (table, c);
+      c = FETCH_CHAR (from);
+      code = SYNTAX (table, c);
 
-	/* is this a 1-char comment end sequence? if so, try
-	   to see if style matches previously extracted mask */
-	if (code == Sendcomment)
+      /* is this a 1-char comment end sequence? if so, try
+	 to see if style matches previously extracted mask */
+      if (code == Sendcomment)
 	{
-	    styles_match_p = SYNTAX_STYLES_MATCH_1CHAR_P (table, c, mask);
+	  styles_match_p = SYNTAX_STYLES_MATCH_1CHAR_P (table, c, mask);
 	}
 
-	/* otherwise, is this a 2-char comment end sequence?
-	   if so, back up, and see if style matches previously
-	   extracted mask */
-	else if (from > stop
-		 && SYNTAX_END_P (table, CHAR_AT (from-1), c))
+      /* otherwise, is this a 2-char comment end sequence?
+	 if so, back up, and see if style matches previously
+	 extracted mask */
+      else if (from > stop
+	       && SYNTAX_END_P (table, FETCH_CHAR (from-1), c))
 	{
-	    code = Sendcomment;
-	    styles_match_p =
-	      SYNTAX_STYLES_MATCH_END_P (table, CHAR_AT (from-1), c, mask);
-
-	    from--;
+	  code = Sendcomment;
+	  styles_match_p = SYNTAX_STYLES_MATCH_END_P (table,
+						      FETCH_CHAR (from-1),
+						      c, mask);
+	  from--;
 	}
 			
-	/* or are we looking at a 1-char comment start sequence
-	   of the style matching mask? */
-	else if (code == Scomment
-		 && SYNTAX_STYLES_MATCH_1CHAR_P (table, c, mask))
+      /* or are we looking at a 1-char comment start sequence
+	 of the style matching mask? */
+      else if (code == Scomment
+	       && SYNTAX_STYLES_MATCH_1CHAR_P (table, c, mask))
 	{
-	    styles_match_p = 1;
+	  styles_match_p = 1;
 	}
 		    
-	/* or possibly, a 2-char comment start sequence */
-	else if (from > stop
-		 && SYNTAX_STYLES_MATCH_START_P (table, CHAR_AT (from-1),
-                                                 c, mask))
+      /* or possibly, a 2-char comment start sequence */
+      else if (from > stop
+	       && SYNTAX_STYLES_MATCH_START_P (table, FETCH_CHAR (from-1),
+					       c, mask))
 	{
-	    code = Scomment;
-	    from--;
-	    styles_match_p = 1;
+	  code = Scomment;
+	  from--;
+	  styles_match_p = 1;
 	}
 
-	/* Ignore escaped characters.  */
-	if (char_quoted (from))
-	    continue;
+      /* Ignore escaped characters.  */
+      if (char_quoted (from))
+	continue;
 
-	/* Track parity of quotes.  */
-	if (code == Sstring)
+      /* Track parity of quotes.  */
+      if (code == Sstring)
 	{
-          parity ^= 1;
-          if (my_stringend == 0)
-            my_stringend = c;
-          /* If we have two kinds of string delimiters.
-             There's no way to grok this scanning backwards.  */
-          else if (my_stringend != c)
-            string_lossage = 1;
+	  parity ^= 1;
+	  if (my_stringend == 0)
+	    my_stringend = c;
+	  /* If we have two kinds of string delimiters.
+	     There's no way to grok this scanning backwards.  */
+	  else if (my_stringend != c)
+	    string_lossage = 1;
 	}
 
-	/* Record comment-starters according to that
-	   quote-parity to the comment-end.  */
-	if (code == Scomment && styles_match_p)
+      /* Record comment-starters according to that
+	 quote-parity to the comment-end.  */
+      if (code == Scomment && styles_match_p)
 	{
-	    comstart_parity = parity;
-	    comstart_pos = from;
+	  comstart_parity = parity;
+	  comstart_pos = from;
 	}
 
-	/* If we find another earlier comment-ender,
-	   any comment-starts earier than that don't count
-	   (because they go with the earlier comment-ender).  */
-	if (code == Sendcomment && styles_match_p)
-	    break;
+      /* If we find another earlier comment-ender,
+	 any comment-starts earlier than that don't count
+	 (because they go with the earlier comment-ender).  */
+      if (code == Sendcomment && styles_match_p)
+	break;
 
-	/* Assume a defun-start point is outside of strings.  */
-	if (code == Sopen
-	    && (from == stop || CHAR_AT (from - 1) == '\n'))
-	    break;
+      /* Assume a defun-start point is outside of strings.  */
+      if (code == Sopen
+	  && (from == stop || FETCH_CHAR (from - 1) == '\n'))
+	break;
     }
 
-    if (comstart_pos == 0)
-	from = comment_end;
-    /* If the earliest comment starter
-       is followed by uniform paired string quotes or none,
-       we know it can't be inside a string
-       since if it were then the comment ender would be inside one.
-       So it does start a comment.  Skip back to it.  */
-    else if (comstart_parity == 0 && !string_lossage)
-	from = comstart_pos;
-    else
+  if (comstart_pos == 0)
+    from = comment_end;
+  /* If the earliest comment starter
+     is followed by uniform paired string quotes or none,
+     we know it can't be inside a string
+     since if it were then the comment ender would be inside one.
+     So it does start a comment.  Skip back to it.  */
+  else if (comstart_parity == 0 && !string_lossage)
+    from = comstart_pos;
+  else
     {
       /* We had two kinds of string delimiters mixed up
 	 together.  Decode this going forwards.
@@ -471,15 +467,15 @@ find_start_of_comment (from, stop, mask)
 
       struct lisp_parse_state state;
       scan_sexps_forward (&state, find_defun_start (comment_end),
-			  comment_end - 1, -10000, 0, Qnil);
+			  comment_end - 1, -10000, 0, Qnil, 0);
       if (state.incomment)
 	from = state.comstart;
       else
-      /* We can't grok this as a comment; scan it normally.  */
+	/* We can't grok this as a comment; scan it normally.  */
 	from = comment_end;
     }
-    return from;
-}  
+  return from;
+}
 
 static int
 find_end_of_comment (from, stop, mask)
@@ -494,7 +490,7 @@ find_end_of_comment (from, stop, mask)
 	{
 	  return -1;
 	}
-      c = CHAR_AT (from);
+      c = FETCH_CHAR (from);
       if (SYNTAX (table, c) == Sendcomment
 	  && SYNTAX_STYLES_MATCH_1CHAR_P (table, c, mask))
 	/* we have encountered a comment end of the same style
@@ -504,7 +500,7 @@ find_end_of_comment (from, stop, mask)
 
       from++;
       if (from < stop
-	  && SYNTAX_STYLES_MATCH_END_P (table, c, CHAR_AT (from), mask))
+	  && SYNTAX_STYLES_MATCH_END_P (table, c, FETCH_CHAR (from), mask))
 	/* we have encountered a comment end of the same style
 	   as the comment sequence which began this comment
 	   section */
@@ -514,27 +510,25 @@ find_end_of_comment (from, stop, mask)
 }
 
 
-int parse_sexp_ignore_comments;
-
 DEFUN ("forward-comment", Fforward_comment, Sforward_comment, 1, 1, 0,
   "Move forward across up to N comments.  If N is negative, move backward.\n\
-Set point to the far end of the last comment found.\n\
 Stop scanning if we find something other than a comment or whitespace.\n\
+Set point to where scanning stops.\n\
 If N comments are found as expected, with nothing except whitespace\n\
-between them, return t; otherwise return nil.  Point is set in either case.")
-  (cntarg)
-     Lisp_Object cntarg;
+between them, return t; otherwise return nil.\n\
+Point is set in either case.")
+     (n)
+     Lisp_Object n;
 {
   register int from;
   register int stop;
   register int c;
   register enum syntaxcode code;
   int count;
-  int mask = 0;			/* mask for finding matching comment style */
   Lisp_Object table = current_buffer->syntax_table;
 
-  CHECK_FIXNUM (cntarg, 0);
-  count = XINT (cntarg);
+  CHECK_FIXNUM (n, 0);
+  count = XINT (n);
 
   immediate_quit = 1;
   QUIT;
@@ -546,12 +540,16 @@ between them, return t; otherwise return nil.  Point is set in either case.")
       stop = ZV;
       while (from < stop)
 	{
-	  if (char_quoted (from))
-	    { from++; continue; }
+	  int mask = 0;         /* mask for finding matching comment style */
 
-	  c = CHAR_AT (from);
+	  if (char_quoted (from))
+	    {
+	      from++;
+	      continue;
+	    }
+
+	  c = FETCH_CHAR (from);
 	  code = SYNTAX (table, c);
-	  mask = 0x0;
 
 	  if (code == Scomment)
 	    {
@@ -563,7 +561,8 @@ between them, return t; otherwise return nil.  Point is set in either case.")
 	      mask = SYNTAX_COMMENT_1CHAR_MASK (table, c);
 	    }
 
-	  else if (from < stop && SYNTAX_START_P (table, c, CHAR_AT (from+1)))
+	  else if (from < stop
+		   && SYNTAX_START_P (table, c, FETCH_CHAR (from+1)))
 	    {
 	      /* we have encountered a 2char comment start sequence and we 
 		 are ignoring all text inside comments. we must record
@@ -571,7 +570,7 @@ between them, return t; otherwise return nil.  Point is set in either case.")
 		 only a comment end of the same style actually ends
 		 the comment section */
 	      code = Scomment;
-	      mask = SYNTAX_COMMENT_MASK_START (table, c, CHAR_AT (from+1));
+	      mask = SYNTAX_COMMENT_MASK_START (table, c, FETCH_CHAR (from+1));
 	      from++;
 	    }
 
@@ -592,9 +591,9 @@ between them, return t; otherwise return nil.  Point is set in either case.")
 	      /* We have skipped one comment.  */
 	      break;
 	    }
-	  else if (code != Swhitespace &&
-		   code != Sendcomment &&
-		   code != Scomment )
+	  else if (code != Swhitespace
+		   && code != Sendcomment
+		   && code != Scomment )
 	    {
 	      immediate_quit = 0;
 	      SET_PT (from);
@@ -612,13 +611,17 @@ between them, return t; otherwise return nil.  Point is set in either case.")
       stop = BEGV;
       while (from > stop)
 	{
+          int mask = 0;         /* mask for finding matching comment style */
+
 	  from--;
 	  if (char_quoted (from))
-	    { from--; continue; }
+	    {
+	      from--;
+	      continue;
+	    }
 	      
-	  c = CHAR_AT (from);
+	  c = FETCH_CHAR (from);
 	  code = SYNTAX (table, c);
-	  mask = 0x0;
 
 	  if (code == Sendcomment)
 	    {
@@ -629,14 +632,14 @@ between them, return t; otherwise return nil.  Point is set in either case.")
 	    }
 
 	  else if (from > stop
-		   && SYNTAX_END_P (table, CHAR_AT (from-1), c)
+		   && SYNTAX_END_P (table, FETCH_CHAR (from - 1), c)
 		   && !char_quoted (from - 1))
 	    {
-	      /* we must record the comment style encountered so that
+	      /* We must record the comment style encountered so that
 		 later, we can match only the proper comment begin
-		 sequence of the same style */
+		 sequence of the same style.  */
 	      code = Sendcomment;
-	      mask = SYNTAX_COMMENT_MASK_END (CHAR_AT (from-1), c);
+	      mask = SYNTAX_COMMENT_MASK_END (FETCH_CHAR (from - 1), c);
 	      from--;
 	    }
 
@@ -646,9 +649,9 @@ between them, return t; otherwise return nil.  Point is set in either case.")
  	      break;
             }
 
-	  else if (code != Swhitespace &&
-		   SYNTAX (table, c) != Scomment &&
-		   SYNTAX (table, c) != Sendcomment)
+	  else if (code != Swhitespace
+		   && SYNTAX (table, c) != Scomment
+		   && SYNTAX (table, c) != Sendcomment)
 	    {
 	      immediate_quit = 0;
 	      SET_PT (from + 1);
@@ -676,7 +679,6 @@ scan_lists (from, count, depth, sexpflag)
   int mathexit = 0;
   register enum syntaxcode code;
   int min_depth = depth;    /* Err out if depth gets less than this. */
-  int mask = 0x0;	    /* comment style mask */
   Lisp_Object table = current_buffer->syntax_table;
 
   if (depth > 0) min_depth = 0;
@@ -689,7 +691,9 @@ scan_lists (from, count, depth, sexpflag)
       stop = ZV;
       while (from < stop)
 	{
-	  c = CHAR_AT (from);
+          int mask = 0;         /* mask for finding matching comment style */
+
+	  c = FETCH_CHAR (from);
 	  code = SYNTAX (table, c);
 	  from++;
 
@@ -701,7 +705,7 @@ scan_lists (from, count, depth, sexpflag)
 
 	  /* else, a 2-char comment start sequence? */
 	  else if (from < stop
-		   && SYNTAX_START_P (table, c, CHAR_AT (from))
+		   && SYNTAX_START_P (table, c, FETCH_CHAR (from))
 		   && parse_sexp_ignore_comments)
 	    {
 	      /* we have encountered a comment start sequence and we 
@@ -710,7 +714,7 @@ scan_lists (from, count, depth, sexpflag)
 		 only a comment end of the same style actually ends
 		 the comment section */
 	      code = Scomment;
-	      mask = SYNTAX_COMMENT_MASK_START (table, c, CHAR_AT (from));
+	      mask = SYNTAX_COMMENT_MASK_START (table, c, FETCH_CHAR (from));
 	      from++;
 	    }
 	  
@@ -730,7 +734,7 @@ scan_lists (from, count, depth, sexpflag)
 	      /* This word counts as a sexp; return at end of it. */
 	      while (from < stop)
 		{
-		  switch (SYNTAX (table, CHAR_AT (from)))
+		  switch (SYNTAX (table, FETCH_CHAR (from)))
 		    {
 		    case Scharquote:
 		    case Sescape:
@@ -749,7 +753,8 @@ scan_lists (from, count, depth, sexpflag)
 	      goto done;
 
 	    case Scomment:
-	      if (!parse_sexp_ignore_comments) break;
+	      if (!parse_sexp_ignore_comments)
+		break;
 	      {
 		int newfrom = find_end_of_comment (from, stop, mask);
 		if (newfrom < 0)
@@ -765,7 +770,7 @@ scan_lists (from, count, depth, sexpflag)
 	    case Smath:
 	      if (!sexpflag)
 		break;
-	      if (from != stop && c == CHAR_AT (from))
+	      if (from != stop && c == FETCH_CHAR (from))
 		from++;
 	      if (mathexit)
 		{
@@ -787,16 +792,18 @@ scan_lists (from, count, depth, sexpflag)
 
 	    case Sstring:
               {
-                int ch = CHAR_AT (from - 1);
+                int ch = FETCH_CHAR (from - 1);
                 unsigned char stringterm = SYNTAX_MATCH (table, ch);
                 if (stringterm == 0)
                   stringterm = ch;
               
                 while (1)
 		{
-		  if (from >= stop) goto lose;
-		  if (CHAR_AT (from) == stringterm) break;
-		  switch (SYNTAX (table, CHAR_AT (from)))
+		  if (from >= stop)
+		    goto lose;
+		  if (FETCH_CHAR (from) == stringterm)
+		    break;
+		  switch (SYNTAX (table, FETCH_CHAR (from)))
                   {
                   case Scharquote:
                   case Sescape:
@@ -833,14 +840,15 @@ scan_lists (from, count, depth, sexpflag)
       stop = BEGV;
       while (from > stop)
 	{
+          int mask = 0;         /* mask for finding matching comment style */
+
 	  from--;
           quoted = char_quoted (from);
 	  if (quoted)
 	    from--;
 
-	  c = CHAR_AT (from);
+	  c = FETCH_CHAR (from);
 	  code = SYNTAX (table, c);
-	  mask = 0x0;
 
 	  if (code == Sendcomment && parse_sexp_ignore_comments)
 	    {
@@ -851,7 +859,7 @@ scan_lists (from, count, depth, sexpflag)
 	    }
 
 	  else if (from > stop
-		   && SYNTAX_END_P (table, CHAR_AT (from-1), c)
+		   && SYNTAX_END_P (table, FETCH_CHAR (from-1), c)
 		   && !char_quoted (from - 1)
 		   && parse_sexp_ignore_comments)
 	    {
@@ -859,7 +867,7 @@ scan_lists (from, count, depth, sexpflag)
 		 later, we can match only the proper comment begin
 		 sequence of the same style */
 	      code = Sendcomment;
-	      mask = SYNTAX_COMMENT_MASK_END (CHAR_AT (from-1), c);
+	      mask = SYNTAX_COMMENT_MASK_END (FETCH_CHAR (from - 1), c);
 	      from--;
 	    }
 
@@ -879,9 +887,9 @@ scan_lists (from, count, depth, sexpflag)
 		  if (quoted)
 		    from--;
 		  if (! (quoted
-                         || SYNTAX (table, CHAR_AT (from - 1)) == Sword
-			 || SYNTAX (table, CHAR_AT (from - 1)) == Ssymbol
-			 || SYNTAX (table, CHAR_AT (from - 1)) == Squote))
+                         || SYNTAX (table, FETCH_CHAR (from - 1)) == Sword
+			 || SYNTAX (table, FETCH_CHAR (from - 1)) == Ssymbol
+			 || SYNTAX (table, FETCH_CHAR (from - 1)) == Squote))
             	    goto done2;
 		  from--;
 		}
@@ -890,7 +898,7 @@ scan_lists (from, count, depth, sexpflag)
 	    case Smath:
 	      if (!sexpflag)
 		break;
-	      if (from != stop && c == CHAR_AT (from - 1))
+	      if (from != stop && c == FETCH_CHAR (from - 1))
 		from--;
 	      if (mathexit)
 		{
@@ -913,12 +921,11 @@ scan_lists (from, count, depth, sexpflag)
 	    case Sendcomment:
 	      if (parse_sexp_ignore_comments)
 		from = find_start_of_comment (from, stop, mask);
-
 	      break;
 
 	    case Sstring:
               {
-                int ch = CHAR_AT (from);
+                int ch = FETCH_CHAR (from);
                 unsigned char stringterm = SYNTAX_MATCH (table, ch);
                 if (stringterm == 0)
                   stringterm = ch;
@@ -927,7 +934,7 @@ scan_lists (from, count, depth, sexpflag)
 		{
 		  if (from == stop) goto lose;
 		  if (!char_quoted (from - 1)
- 		      && stringterm == CHAR_AT (from - 1))
+ 		      && stringterm == FETCH_CHAR (from - 1))
 		    break;
 		  from--;
 		}
@@ -968,7 +975,7 @@ char_quoted (pos)
   Lisp_Object table = current_buffer->syntax_table;
 
   while (pos > beg
-	 && ((code = SYNTAX (table, CHAR_AT (pos - 1))) == Scharquote
+	 && ((code = SYNTAX (table, FETCH_CHAR (pos - 1))) == Scharquote
 	     || code == Sescape))
     pos--, quoted = !quoted;
   return quoted;
@@ -1025,12 +1032,12 @@ This includes chars with \"quote\" or \"prefix\" syntax (' or p).")
   ()
 {
   int beg = BEGV;
-  int pos = point;
+  int pos = PT;
   Lisp_Object table = current_buffer->syntax_table;
 
   while (pos > beg && !char_quoted (pos - 1)
-	 && (SYNTAX (table, CHAR_AT (pos - 1)) == Squote
-	     || SYNTAX_PREFIX (table, CHAR_AT (pos - 1))))
+	 && (SYNTAX (table, FETCH_CHAR (pos - 1)) == Squote
+	     || SYNTAX_PREFIX (table, FETCH_CHAR (pos - 1))))
     pos--;
 
   SET_PT (pos);
@@ -1039,15 +1046,17 @@ This includes chars with \"quote\" or \"prefix\" syntax (' or p).")
 }
 
 /* Parse forward from FROM to END,
-   assuming that FROM is the start of a function, 
-   and return a description of the state of the parse at END. */
+   assuming that FROM has state OLDSTATE (nil means FROM is start of function),
+   and return a description of the state of the parse at END.
+   If STOPBEFORE is nonzero, stop at the start of an atom.
+   If COMMENTSTOP is nonzero, stop at the start of a comment.  */
 
 static void
-scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
-     struct lisp_parse_state *stateptr;
-     register int from;
-     int end, targetdepth, stopbefore;
-     Lisp_Object oldstate;
+scan_sexps_forward (struct lisp_parse_state *stateptr,
+		    int from, int end, 
+		    int targetdepth, int stopbefore,
+		    Lisp_Object oldstate,
+		    int commentstop)
 {
   struct lisp_parse_state state;
 
@@ -1073,12 +1082,12 @@ scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
       depth = 0;
       state.instring = -1;
       state.incomment = 0;
-      state.comstyle = 0;	/* default comstyle is a */
+      state.comstyle = 0;	/* comment style a by default */
       mask = SYNTAX_COMMENT_STYLE_A;
     }
   else
     {
-      tem = Fcar (oldstate);
+      tem = Fcar (oldstate);    /* elt 0, depth */
       if (!NILP (tem))
 	depth = XINT (tem);
       else
@@ -1087,23 +1096,23 @@ scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
       oldstate = Fcdr (oldstate);
       oldstate = Fcdr (oldstate);
       oldstate = Fcdr (oldstate);
-      tem = Fcar (oldstate);
+      tem = Fcar (oldstate);    /* elt 3, instring */
       state.instring = !NILP (tem) ? XINT (tem) : -1;
 
-      oldstate = Fcdr (oldstate);
+      oldstate = Fcdr (oldstate); /* elt 4, incomment */
       tem = Fcar (oldstate);
       state.incomment = !NILP (tem);
 
       oldstate = Fcdr (oldstate);
-      tem = Fcar (oldstate);
+      tem = Fcar (oldstate);    /* elt 5, follows-quote */
       start_quoted = !NILP (tem);
 
-      /* if eighth element of the list is nil, we are in comment style
+      /* if the eighth element of the list is nil, we are in comment style
 	 a. if it is non-nil, we are in comment style b */
       oldstate = Fcdr (oldstate);
       oldstate = Fcdr (oldstate);
       oldstate = Fcdr (oldstate);
-      tem = Fcar (oldstate);
+      tem = Fcar (oldstate);    /* elt 8, comment style a */
       state.comstyle = !NILP (tem);
       mask = state.comstyle ? SYNTAX_COMMENT_STYLE_B : SYNTAX_COMMENT_STYLE_A;
     }
@@ -1125,7 +1134,7 @@ scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
 
   while (from < end)
     {
-      code = SYNTAX (table, CHAR_AT (from));
+      code = SYNTAX (table, FETCH_CHAR (from));
       from++;
 
       if (code == Scomment)
@@ -1133,26 +1142,27 @@ scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
 	  /* record the comment style we have entered so that only the
 	     comment-ender sequence (or single char) of the same style
 	     actually terminates the comment section. */
-	  mask = SYNTAX_COMMENT_1CHAR_MASK (table, CHAR_AT (from-1));
+	  mask = SYNTAX_COMMENT_1CHAR_MASK (table, FETCH_CHAR (from-1));
 	  state.comstyle = (mask == SYNTAX_COMMENT_STYLE_B);
-	  state.comstart = from-1;
+	  state.comstart = from - 1;
 	}
       
       else if (from < end &&
-	       SYNTAX_START_P (table, CHAR_AT (from-1), CHAR_AT (from)))
+	       SYNTAX_START_P (table, FETCH_CHAR (from-1), FETCH_CHAR (from)))
 	{
 	  /* Record the comment style we have entered so that only
 	     the comment-end sequence of the same style actually
 	     terminates the comment section.  */
 	  code = Scomment;
 	  mask = SYNTAX_COMMENT_MASK_START (table,
-                                            CHAR_AT (from-1), CHAR_AT (from));
+                                            FETCH_CHAR (from-1),
+					    FETCH_CHAR (from));
 	  state.comstyle = (mask == SYNTAX_COMMENT_STYLE_B);
 	  state.comstart = from-1;
 	  from++;
 	}
 
-      if (SYNTAX_PREFIX (table, CHAR_AT (from - 1)))
+      if (SYNTAX_PREFIX (table, FETCH_CHAR (from - 1)))
 	continue;
       switch (code)
 	{
@@ -1172,7 +1182,7 @@ scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
 	symstarted:
 	  while (from < end)
 	    {
-	      switch (SYNTAX (table, CHAR_AT (from)))
+	      switch (SYNTAX (table, FETCH_CHAR (from)))
 		{
 		case Scharquote:
 		case Sescape:
@@ -1195,6 +1205,8 @@ scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
 	case Scomment:
 	  state.incomment = 1;
 	startincomment:
+	  if (commentstop)
+	    goto done;
 	  {
 	    int newfrom = find_end_of_comment (from, end, mask);
 	    if (newfrom < 0)
@@ -1237,7 +1249,7 @@ scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
             int ch;
             if (stopbefore) goto stop; /* this arg means stop at sexp start */
             curlevel->last = from - 1;
-            ch = CHAR_AT (from - 1);
+            ch = FETCH_CHAR (from - 1);
             state.instring = SYNTAX_MATCH (table, ch);
             if (state.instring == 0) state.instring = ch;
           }
@@ -1245,8 +1257,8 @@ scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
 	  while (1)
 	    {
 	      if (from >= end) goto done;
-	      if (CHAR_AT (from) == state.instring) break;
-	      switch (SYNTAX (table, CHAR_AT (from)))
+	      if (FETCH_CHAR (from) == state.instring) break;
+	      switch (SYNTAX (table, FETCH_CHAR (from)))
 		{
 		case Scharquote:
 		case Sescape:
@@ -1301,35 +1313,37 @@ scan_sexps_forward (stateptr, from, end, targetdepth, stopbefore, oldstate)
    for make-docfile to see.  We cannot put this in the real DEFUN
    due to limits in the Unix cpp.
 
-DEFUN ("parse-partial-sexp", Ffoo, Sfoo, 2, 5, 0,
+DEFUN ("parse-partial-sexp", Ffoo, Sfoo, 2, 6, 0,
   "Parse Lisp syntax starting at FROM until TO; return status of parse at TO.\n\
 Parsing stops at TO or when certain criteria are met;\n\
  point is set to where parsing stops.\n\
 If fifth arg STATE is omitted or nil,\n\
  parsing assumes that FROM is the beginning of a function.\n\
 Value is a list of eight elements describing final state of parsing:\n\
- 1. depth in parens.\n\
- 2. character address of start of innermost containing list; nil if none.\n\
- 3. character address of start of last complete sexp terminated.\n\
- 4. non-nil if inside a string.\n\
+ 0. depth in parens.\n\
+ 1. character address of start of innermost containing list; nil if none.\n\
+ 2. character address of start of last complete sexp terminated.\n\
+ 3. non-nil if inside a string.\n\
     (it is the character that will terminate the string.)\n\
- 5. t if inside a comment.\n\
- 6. t if following a quote character.\n\
- 7. the minimum paren-depth encountered during this scan.\n\
- 8. nil if in comment style a, or not in a comment; t if in comment style b\n\
+ 4. t if inside a comment.\n\
+ 5. t if following a quote character.\n\
+ 6. the minimum paren-depth encountered during this scan.\n\
+ 7. nil if in comment style a, or not in a comment; t if in comment style b\n\
 If third arg TARGETDEPTH is non-nil, parsing stops if the depth\n\
 in parentheses becomes equal to TARGETDEPTH.\n\
 Fourth arg STOPBEFORE non-nil means stop when come to\n\
  any character that starts a sexp.\n\
-Fifth arg STATE is a seven-list like what this function returns.\n\
-It is used to initialize the state of the parse.")
-  (from, to, targetdepth, stopbefore, oldstate)
+Fifth arg STATE is an eight-element list like what this function returns.\n\
+It is used to initialize the state of the parse.  Its second and third\n\
+elements are ignored.\n\
+Sixth args COMMENTSTOP non-nil means stop at the start of a comment.")
+  (from, to, targetdepth, stopbefore, oldstate, commentstop)
 */
 
-DEFUN ("parse-partial-sexp", Fparse_partial_sexp, Sparse_partial_sexp, 2, 5, 0,
+DEFUN ("parse-partial-sexp", Fparse_partial_sexp, Sparse_partial_sexp, 2, 6, 0,
   0 /* See immediately above */)
-  (from, to, targetdepth, stopbefore, oldstate)
-     Lisp_Object from, to, targetdepth, stopbefore, oldstate;
+  (from, to, targetdepth, stopbefore, oldstate, commentstop)
+     Lisp_Object from, to, targetdepth, stopbefore, oldstate, commentstop;
 {
   struct lisp_parse_state state;
   int target;
@@ -1344,19 +1358,29 @@ DEFUN ("parse-partial-sexp", Fparse_partial_sexp, Sparse_partial_sexp, 2, 5, 0,
 
   validate_region (&from, &to);
   scan_sexps_forward (&state, XINT (from), XINT (to),
-		      target, !NILP (stopbefore), oldstate);
+		      target, !NILP (stopbefore), oldstate,
+		      !NILP (commentstop));
 
   SET_PT (state.location);
-  
-  return Fcons (make_number (state.depth),
-	   Fcons (state.prevlevelstart < 0 ? Qnil : make_number (state.prevlevelstart),
-	     Fcons (state.thislevelstart < 0 ? Qnil : make_number (state.thislevelstart),
-	       Fcons (state.instring >= 0 ? make_number (state.instring) : Qnil,
-		 Fcons (state.incomment ? Qt : Qnil,
-		   Fcons (state.quoted ? Qt : Qnil,
-		     Fcons (make_number (state.mindepth), 
-		       Fcons (state.comstyle ? Qt : Qnil,
-			    Qnil))))))));
+
+  {
+    /*
+     * This junk is necessary because of a bug in SparcWorks cc 2.0.1.  It
+     * doesn't handle functions as arguments to other functions very well.
+     */
+    Lisp_Object retval[8];
+
+    retval[0] = make_number (state.depth);
+    retval[1] = ((state.prevlevelstart < 0) ? Qnil : make_number (state.prevlevelstart));
+    retval[2] = ((state.thislevelstart < 0) ? Qnil : make_number (state.thislevelstart));
+    retval[3] = ((state.instring >= 0) ? make_number (state.instring) : Qnil);
+    retval[4] = ((state.incomment) ? Qt : Qnil);
+    retval[5] = ((state.quoted) ? Qt : Qnil);
+    retval[6] = make_number (state.mindepth);
+    retval[7] = ((state.comstyle) ? Qt : Qnil);
+
+    return (Flist (8, retval));
+  }
 }
 
 void
@@ -1393,7 +1417,7 @@ init_syntax_once ()
   v->contents['\\'] = make_number ((int) Sescape);
 
   {
-    const char *p;
+    CONST char *p;
     for (p = "_-+*/&|<>="; *p; p++)
       v->contents[(int) *p] = make_number ((int) Ssymbol);
 
