@@ -1,6 +1,4 @@
 /* Unexec for MIPS (including IRIS4D).
-   Copyright (C) 1988, 1992 Free Software Foundation, Inc.
-
    Note that the GNU project considers support for MIPS operation
    a peripheral activity which should not be allowed to divert effort
    from development of the GNU system.  Changes in this code will be
@@ -8,23 +6,23 @@
    we don't plan to think about it, or about whether other Emacs
    maintenance might break it.
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2, or (at your option)
-    any later version.
+   Copyright (C) 1988, 1992, 1993 Free Software Foundation, Inc.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+This file is part of GNU Emacs.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
 
-In other words, you are welcome to use, share and improve this program.
-You are forbidden to forbid anyone else to use, share and improve
-what you give them.   Help stamp out software-hoarding!  */
+GNU Emacs is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "config.h"
 #include <sys/types.h>
@@ -37,7 +35,7 @@ what you give them.   Help stamp out software-hoarding!  */
 #include <scnhdr.h>
 #include <sym.h>
 
-#if defined(IRIS_4D) || defined(sony)
+#if defined (IRIS_4D) || defined (sony)
 #include "getpagesize.h"
 #include <fcntl.h>
 #endif
@@ -47,17 +45,17 @@ static void mark_x ();
 
 #define READ(_fd, _buffer, _size, _error_message, _error_arg) \
 	errno = EEOF; \
-	if (read(_fd, _buffer, _size) != _size) \
-	  fatal_unexec(_error_message, _error_arg);
+	if (read (_fd, _buffer, _size) != _size) \
+	  fatal_unexec (_error_message, _error_arg);
 
 #define WRITE(_fd, _buffer, _size, _error_message, _error_arg) \
-	if (write(_fd, _buffer, _size) != _size) \
-	  fatal_unexec(_error_message, _error_arg);
+	if (write (_fd, _buffer, _size) != _size) \
+	  fatal_unexec (_error_message, _error_arg);
 
 #define SEEK(_fd, _position, _error_message, _error_arg) \
 	errno = EEOF; \
-	if (lseek(_fd, _position, L_SET) != _position) \
-	  fatal_unexec(_error_message, _error_arg);
+	if (lseek (_fd, _position, L_SET) != _position) \
+	  fatal_unexec (_error_message, _error_arg);
 
 extern int errno;
 extern int sys_nerr;
@@ -114,7 +112,7 @@ unexec (new_name, a_name, data_start, bss_start, entry_address)
       && hdr.fhdr.f_magic != (MIPSELMAGIC | 1)
       && hdr.fhdr.f_magic != (MIPSEBMAGIC | 1))
     {
-      fprintf(stderr,
+      fprintf (stderr,
 	      "unexec: input file magic number is %x, not %x, %x, %x or %x.\n",
 	      hdr.fhdr.f_magic,
 	      MIPSELMAGIC, MIPSEBMAGIC,
@@ -125,68 +123,66 @@ unexec (new_name, a_name, data_start, bss_start, entry_address)
   if (hdr.fhdr.f_magic != MIPSELMAGIC
       && hdr.fhdr.f_magic != MIPSEBMAGIC)
     {
-      fprintf(stderr, "unexec: input file magic number is %x, not %x or %x.\n",
+      fprintf (stderr, "unexec: input file magic number is %x, not %x or %x.\n",
 	      hdr.fhdr.f_magic, MIPSELMAGIC, MIPSEBMAGIC);
-      exit(1);
+      exit (1);
     }
 #endif /* not MIPS2 */
-  if (hdr.fhdr.f_opthdr != sizeof(hdr.aout))
+  if (hdr.fhdr.f_opthdr != sizeof (hdr.aout))
     {
-      fprintf(stderr, "unexec: input a.out header is %d bytes, not %d.\n",
-	      hdr.fhdr.f_opthdr, sizeof(hdr.aout));
-      exit(1);
+      fprintf (stderr, "unexec: input a.out header is %d bytes, not %d.\n",
+               hdr.fhdr.f_opthdr, sizeof (hdr.aout));
+      exit (1);
     }
   if (hdr.aout.magic != ZMAGIC)
     {
-      fprintf(stderr, "unexec: input file a.out magic number is %o, not %o.\n",
+      fprintf (stderr, "unexec: input file a.out magic number is %o, not %o.\n",
 	      hdr.aout.magic, ZMAGIC);
-      exit(1);
+      exit (1);
     }
 
 /*
  * The headers are no longer in text-init-rdata-data order. Therefore we 
  * have to do a linear search for each section in the section list.
  */
-#define CHECK_SCNHDR(ptr, name, flags) \
-  i = 0; ptr = NULL; \
-  while(i < hdr.fhdr.f_nscns && !ptr) { \
-  if (strcmp(hdr.section[i].s_name, name) == 0) { \
-    if (hdr.section[i].s_flags != flags) { \
-      fprintf(stderr, "unexec: %x flags where %x expected in %s section.\n", \
-	      hdr.section[i].s_flags, flags, name); \
-    } \
-    ptr = hdr.section + i; \
-  } \
-  i += 1; \
+#define CHECK_SCNHDR(ptr, name, flags)					\
+  ptr = NULL;								\
+  for (i = 0; i < hdr.fhdr.f_nscns && !ptr; i++)			\
+    if (strcmp (hdr.section[i].s_name, name) == 0)			\
+      {									\
+        if (hdr.section[i].s_flags != flags)				\
+        fprintf (stderr, "unexec: %x flags (%x expected) in %s section.\n", \
+                 hdr.section[i].s_flags, flags, name);			\
+        ptr = hdr.section + i;						\
   }
 
-  CHECK_SCNHDR(text_section,  _TEXT,  STYP_TEXT);
-  CHECK_SCNHDR(init_section,  _INIT,  STYP_INIT);
-  CHECK_SCNHDR(rdata_section, _RDATA, STYP_RDATA);
-  CHECK_SCNHDR(data_section,  _DATA,  STYP_DATA);
+  CHECK_SCNHDR (text_section,  _TEXT,  STYP_TEXT);
+  CHECK_SCNHDR (init_section,  _INIT,  STYP_INIT);
+  CHECK_SCNHDR (rdata_section, _RDATA, STYP_RDATA);
+  CHECK_SCNHDR (data_section,  _DATA,  STYP_DATA);
 #ifdef _LIT8
-  CHECK_SCNHDR(lit8_section,  _LIT8,  STYP_LIT8);
-  CHECK_SCNHDR(lit4_section,  _LIT4,  STYP_LIT4);
+  CHECK_SCNHDR (lit8_section,  _LIT8,  STYP_LIT8);
+  CHECK_SCNHDR (lit4_section,  _LIT4,  STYP_LIT4);
 #endif /* _LIT8 */
-  CHECK_SCNHDR(sdata_section, _SDATA, STYP_SDATA);
-  CHECK_SCNHDR(sbss_section,  _SBSS,  STYP_SBSS);
-  CHECK_SCNHDR(bss_section,   _BSS,   STYP_BSS);
+  CHECK_SCNHDR (sdata_section, _SDATA, STYP_SDATA);
+  CHECK_SCNHDR (sbss_section,  _SBSS,  STYP_SBSS);
+  CHECK_SCNHDR (bss_section,   _BSS,   STYP_BSS);
 #if 0 /* Apparently this error check goes off on irix 3.3,
 	 but it doesn't indicate a real problem.  */
   if (i != hdr.fhdr.f_nscns)
-    fprintf(stderr, "unexec: %d sections found instead of %d.\n",
+    fprintf (stderr, "unexec: %d sections found instead of %d.\n",
 	    i, hdr.fhdr.f_nscns);
 #endif
 
   text_section->s_scnptr = 0;
 
-  pagesize = getpagesize();
-  brk = (sbrk(0) + pagesize - 1) & (-pagesize);
+  pagesize = getpagesize ();
+  brk = (sbrk (0) + pagesize - 1) & (-pagesize);
   hdr.aout.dsize = brk - DATA_START;
   hdr.aout.bsize = 0;
   if (entry_address == 0)
     {
-      extern DEFAULT_ENTRY_ADDRESS();
+      extern DEFAULT_ENTRY_ADDRESS ();
       hdr.aout.entry = (unsigned)DEFAULT_ENTRY_ADDRESS;
     }
   else
@@ -195,7 +191,7 @@ unexec (new_name, a_name, data_start, bss_start, entry_address)
   hdr.aout.bss_start = hdr.aout.data_start + hdr.aout.dsize;
   rdata_section->s_size = data_start - DATA_START;
 
-  /* adjust start and virtual addresses of rdata_section, too */
+  /* Adjust start and virtual addresses of rdata_section, too. */
   rdata_section->s_vaddr = DATA_START;
   rdata_section->s_paddr = DATA_START;
   rdata_section->s_scnptr = text_section->s_scnptr + hdr.aout.tsize;
@@ -242,15 +238,15 @@ unexec (new_name, a_name, data_start, bss_start, entry_address)
       bss_section->s_scnptr = scnptr;
     }
 
-  WRITE(new, (void *) TEXT_START, hdr.aout.tsize,
+  WRITE (new, (void *) TEXT_START, hdr.aout.tsize,
 	"writing text section to %s", new_name);
-  WRITE(new, (void *) DATA_START, hdr.aout.dsize,
+  WRITE (new, (void *) DATA_START, hdr.aout.dsize,
 	"writing text section to %s", new_name);
 
-  SEEK(old, hdr.fhdr.f_symptr, "seeking to start of symbols in %s", a_name);
+  SEEK (old, hdr.fhdr.f_symptr, "seeking to start of symbols in %s", a_name);
   errno = EEOF;
-  nread = read(old, buffer, BUFSIZE);
-  if (nread < sizeof(HDRR)) fatal_unexec("reading symbols from %s", a_name);
+  nread = read (old, buffer, BUFSIZE);
+  if (nread < sizeof (HDRR)) fatal_unexec ("reading symbols from %s", a_name);
 #define symhdr ((pHDRR)buffer)
   newsyms = hdr.aout.tsize + hdr.aout.dsize;
   symrel = newsyms - hdr.fhdr.f_symptr;
@@ -269,20 +265,20 @@ unexec (new_name, a_name, data_start, bss_start, entry_address)
 #undef symhdr
   do
     {
-      if (write(new, buffer, nread) != nread)
-	fatal_unexec("writing symbols to %s", new_name);
-      nread = read(old, buffer, BUFSIZE);
-      if (nread < 0) fatal_unexec("reading symbols from %s", a_name);
+      if (write (new, buffer, nread) != nread)
+	fatal_unexec ("writing symbols to %s", new_name);
+      nread = read (old, buffer, BUFSIZE);
+      if (nread < 0) fatal_unexec ("reading symbols from %s", a_name);
 #undef BUFSIZE
     } while (nread != 0);
 
-  SEEK(new, 0, "seeking to start of header in %s", new_name);
-  WRITE(new, &hdr, sizeof(hdr),
+  SEEK (new, 0, "seeking to start of header in %s", new_name);
+  WRITE (new, &hdr, sizeof (hdr),
 	"writing header of %s", new_name);
 
-  close(old);
-  close(new);
-  mark_x(new_name);
+  close (old);
+  close (new);
+  mark_x (new_name);
 }
 
 /*
